@@ -317,7 +317,7 @@ df.add_system_transition('TRAVEL_N', 'travel_n_followup_pet',
 
 # does like to travel
 df.add_system_transition('TRAVEL_Y', 'TRAVEL_TIME',
-                       '"I think travelling is pretty amazing, too! To go to new places and see all of the amazing '
+                       '"I think travelling is pretty amazing too! To go to new places and see all of the amazing '
                          'sights is a great experience. So, are you an early-riser during '
                          'a vacation or do you like to sleep in?"'
                        '#GSRET(ask_if_plan)')
@@ -351,7 +351,7 @@ df.add_system_transition('TRAVEL_PET_N', 'TRAVEL_TIME',
 df.add_user_transition('TRAVEL_TIME', 'TRAVEL_TIME_D',
                        '{#IDK,[{[not,care], no difference, whatever, depends, change, changes, [not,matter], sometimes, both}]}')
 df.add_user_transition('TRAVEL_TIME', 'TRAVEL_TIME_E',
-                       '[{former, early, save time, dawn, sun rise, morning, waste, first, [not,sleep,{in,late,past}]}]')
+                       '[{former, early, save time, dawn, sun rise, morning, waste, first, [not,#LEM(sleep),{in,late,past}]}]')
 df.add_user_transition('TRAVEL_TIME', 'TRAVEL_TIME_L',
                        '[{latter, second, late, stay, bed, afternoon, noon, night, nightlife, later, [#NOT(not),sleep,in]}]')
 df.set_error_successor('TRAVEL_TIME', 'TRAVEL_TIME_D')
